﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Windows;
using Infragistics.Olap;
using Infragistics.Olap.Data;
using Infragistics.Olap.FlatData;
using Infragistics.Controls.Grids;
using Infragistics.Documents.Excel;

namespace SampleDynamicData
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow
    {
        private readonly Dictionary<string, int> levelNamesList = new Dictionary<string, int>();

        public MainWindow()
        {
            InitializeComponent();
            pivotGrid.DataSource = LoadData();
            pivotGrid.CellControlAttached += pivotGrid_CellControlAttached;
            
        }

        void pivotGrid_CellControlAttached(object sender, Infragistics.Controls.Grids.PivotCellControlAttachedEventArgs e)
        {
            ICell cell = e.Cell.Data as ICell;
            //if (cell != null && cell.FormattedValue == "0") ;
            // e.Cell.Control.Content = null;
        }

        public FlatDataSource LoadData()
        {
            var sourceTable = new DataTable();

            sourceTable.Columns.Add("RowData", typeof(string));
            sourceTable.Columns.Add("Vehicle Type", typeof(string));
            sourceTable.Columns.Add("VehicleName", typeof(string));
            sourceTable.Columns.Add("Rating", typeof(string));
            sourceTable.Columns.Add("UnitPrice", typeof(int));

            foreach (var elem in Data())
            {
                DataRow dr = sourceTable.NewRow();
                dr["RowData"] = elem[0];
                dr["Vehicle Type"] = elem[1];
                dr["VehicleName"] = elem[2];
                dr["Rating"] = elem[3];
                dr["UnitPrice"] = string.IsNullOrEmpty(elem[4]) ? 0 : Convert.ToInt32(elem[4]);
                sourceTable.Rows.Add(dr);
            }

            sourceTable.AcceptChanges();
            return BuildDataSource(sourceTable);
        }

        //Build classes based from datatable columns
        private FlatDataSource BuildDataSource(DataTable table)
        {
            DynamicTypeBuilder typeBuilder = new DynamicTypeBuilder
            {
                DynamicAssemblyName = "MyAssembly",
                DynamicTypeName = "Data"
            };

            IList<DynamicTypePropertyInfo> properties = new List<DynamicTypePropertyInfo>();
            foreach (DataColumn column in table.Columns)
            {
                DynamicTypePropertyInfo propertyInfo = new DynamicTypePropertyInfo
                {
                    PropertyName = column.ColumnName,
                    PropertyType = (column.DataType == typeof(decimal) ? typeof(decimal?) : column.DataType)
                };

                properties.Add(propertyInfo);
            }

            Type dynamicType = typeBuilder.GenerateType(properties);

            Type listType = typeof(List<>);
            Type genericListType = listType.MakeGenericType(dynamicType);
            IList list = (IList)Activator.CreateInstance(genericListType);

            foreach (DataRow dataRow in table.Rows)
            {
                object myDynamicInstance = Activator.CreateInstance(dynamicType);
                foreach (DataColumn column in table.Columns)
                {
                    PropertyInfo propertyVal = dynamicType.GetProperty(column.ColumnName);
                    if (column.DataType == typeof(decimal))
                        propertyVal.SetValue(myDynamicInstance, dataRow[column].ToString() == "" ? null : dataRow[column], null);
                    if (dataRow[column] != DBNull.Value)
                        propertyVal.SetValue(myDynamicInstance, dataRow[column], null);
                }

                if (!string.IsNullOrEmpty(dataRow[0].ToString().Trim()))
                    list.Add(myDynamicInstance);
            }

            FlatDataSource flatDataSource = new FlatDataSource();

            flatDataSource.DimensionsGenerationMode = DimensionsGenerationMode.Mixed;

            CubeMetadata cubeMetadata = new CubeMetadata();
            cubeMetadata.DisplayName = "My Data";
            cubeMetadata.DataTypeFullName = typeBuilder.DynamicTypeName;  // Same as DynamicTypeName

            DimensionMetadata dm = new DimensionMetadata()
            {
                SourcePropertyName = "Vehicle Type",
                DisplayName = "Vehicle Type",
            };

            HierarchyDescriptor vehicleDescriptor = new HierarchyDescriptor()
            {
                SourcePropertyName = "Vehicle Type",
                HierarchyName = "Vehicle Type",
                HierarchyDisplayName = "Vehicle Type",
            };
            //HierarchyLevelDescriptor allLevel = new HierarchyLevelDescriptor
            //{
            //    LevelName = "All Vehicle Types",
            //};
            //vehicleDescriptor.LevelDescriptors.Add(allLevel);

            HierarchyLevelDescriptor level1 = new HierarchyLevelDescriptor
            {
                LevelName = "Vehicle Type",
                LevelExpressionPath = "Vehicle Type",
            };
            vehicleDescriptor.LevelDescriptors.Add(level1);

            HierarchyLevelDescriptor level2 = new HierarchyLevelDescriptor
            {
                LevelName = "Vehicle Name",
                LevelExpressionPath = "VehicleName",
            };
            vehicleDescriptor.LevelDescriptors.Add(level2);

            HierarchyLevelDescriptor level3 = new HierarchyLevelDescriptor
            {
                LevelName = "Rating",
                LevelExpressionPath = "Rating",
            };

            vehicleDescriptor.LevelDescriptors.Add(level3);
            dm.HierarchyDescriptors.Add(vehicleDescriptor);

            cubeMetadata.DimensionSettings.Add(dm);
            flatDataSource.CubesSettings.Add(cubeMetadata);


            flatDataSource.ItemsSource = list;
            flatDataSource.Cube = DataSourceBase.GenerateInitialCube("Data");
            flatDataSource.Columns = DataSourceBase.GenerateInitialItems("[Vehicle Type].[Vehicle Type]");
            flatDataSource.Rows = DataSourceBase.GenerateInitialItems("[RowData].[RowData]");
            flatDataSource.Measures = DataSourceBase.GenerateInitialItems("UnitPrice");
            flatDataSource.ResultChanged += flatDataSource_ResultChanged;

            return flatDataSource;
        }

        void flatDataSource_ResultChanged(object sender, ResultChangedEventArgs e)
        {
            // if you don't want to show totals, uncomment this code.
            //foreach (PivotDataColumn c in pivotGrid.DataColumns)
            //{
            //    if (c.IsTotal)
            //    {
            //        c.IsVisible = false;
            //    }
            //}
        }

        private List<string[]> Data()
        {
            var lst = new List<string[]>();
            string[] parentData = new string[] { "California", "Honda", "CBR 1000R", "5", null };
            string[] child1 = new string[] { "     Los Angeles", "Honda", "CBR 1000R", "4", "11670" };
            string[] child2 = new string[] { "     Los Angeles", "Honda", "CBR 1000R", "2", "12499" };
            string[] child3 = new string[] { "     Oakland", "Honda", "CBR 650F", "5", "15000" };
            string[] child4 = new string[] { "     Oakland", "Honda", "CBR 650F", "1", "15000" };
            string[] child5 = new string[] { "     Riverside", "Honda", "Interceptor", "3", "15999" };
            string[] child6 = new string[] { "     Riverside", "Honda", "CBR 650F", "2", "8499" };
            string[] child7 = new string[] { "     San Francisco", "Honda", "Interceptor", "5", "6499" };

            string[] parentData1 = new string[] { "Texas", "Yamaha", "R1", "1", null };
            string[] childv1 = new string[] { "     Houston", "Yamaha", "R1", "5", "11670" };
            string[] childv2 = new string[] { "     Houston", "Yamaha", "R1", "4", "12499" };
            string[] childv3 = new string[] { "     Dallas", "Yamaha", "R1", "5", "15000" };
            string[] childv4 = new string[] { "     Dallas", "Yamaha", "R6", "2", "15000" };
            string[] childv5 = new string[] { "     Dallas", "Yamaha", "R6", "3", "15999" };
            string[] childv6 = new string[] { "     Austin", "Yamaha", "R6", "1", "8499" };
            string[] childv7 = new string[] { "     Austin", "Yamaha", "R6", "4", "6499" };

            lst.Add(parentData);
            lst.Add(child1);
            lst.Add(child2);
            lst.Add(child3);
            lst.Add(child4);
            lst.Add(child5);
            lst.Add(child6);
            lst.Add(child7);

            lst.Add(parentData1);
            lst.Add(childv1);
            lst.Add(childv2);
            lst.Add(childv3);
            lst.Add(childv4);
            lst.Add(childv5);
            lst.Add(childv6);
            lst.Add(childv7);

            return lst;
        }

        #region Excel Functionality
        
        
        private void ExportToExcel(object sender, RoutedEventArgs e)
        {
            this.SaveExport(this.CreateWorkbook(this.pivotGrid));
        }

        private Workbook CreateWorkbook(XamPivotGrid pivotGrid)
        {
         
            Boolean CompactLayoutEnabled = pivotGrid.AllowCompactLayout;

            if (CompactLayoutEnabled)
                pivotGrid.AllowCompactLayout = false;

            int currentRow = 12;

            //Calculate the dimensions of the header rows and columns
            int TopHeaderHeight = 0;

            TopHeaderHeight = pivotGrid.GridLayout.PrivateRowsForColumnPanel.Count;

            int LeftHeaderWidth = 0;
            LeftHeaderWidth = pivotGrid.GridLayout.PrivateColumnsForRowPanel.Count;

            //Create the blank workbook and the sheet in it that will contain the PivotGrid data
            Workbook dataWorkbook = new Workbook();
            Worksheet sheet = dataWorkbook.Worksheets.Add("Main Report");

            sheet.DisplayOptions.PanesAreFrozen = true;
            sheet.DisplayOptions.FrozenPaneSettings.FrozenRows = TopHeaderHeight + currentRow;
            sheet.DisplayOptions.FrozenPaneSettings.FrozenColumns = LeftHeaderWidth;

            //Set default width for the columns
            sheet.DefaultColumnWidth = 6000;

            //First column set to 16000
            sheet.Columns[1].Width = 16000;

            int ColumnSpan, ColumnId, RowSpan, RowId;
            string CellValue;

            int i = 1;
            // Build Column Header
            foreach (PivotColumnHeaderCell cell in pivotGrid.GridLayout.ColumnHeaderCells)
            {
                ColumnSpan = cell.RealColumnSpan;
                ColumnId = pivotGrid.GridLayout.Columns
                    .IndexOf(cell.Column as PivotDataColumn)
                           + LeftHeaderWidth;

                RowSpan = cell.RowSpan;
                RowId = pivotGrid.GridLayout
                    .PrivateRowsForColumnPanel.IndexOf(cell.Row as PivotHeaderRow) + currentRow;

                CellValue = cell.Member.Caption;
                if (!levelNamesList.ContainsKey(cell.Member.LevelName))
                {
                    levelNamesList.Add(cell.Member.LevelName, i);
                    i++;
                }
                if (cell.Member.IsTotal && cell.IsToggleVisible == false)
                    CellValue += " " + "Total";

                if (ColumnSpan > 1 || RowSpan > 1)
                {
                    sheet.MergedCellsRegions.Add(
                        RowId,
                        ColumnId,
                        RowId + RowSpan - 1,
                        ColumnId + ColumnSpan - 1
                        );

                    //Outline for Excel
                    for (int j = ColumnId; j <= ColumnId + ColumnSpan - 2; j++)
                    {
                        sheet.Columns[j].OutlineLevel = levelNamesList.FirstOrDefault(x => x.Key.Equals(cell.Member.LevelName)).Value;
                        sheet.Columns[j].Hidden = true;
                    }
               }
                if (CellValue.Equals("N/A"))
                {
                    sheet.Columns[ColumnId].Hidden = true;
                    continue;
                }
                this.SetCellValue(sheet.Rows[RowId].Cells[ColumnId], CellValue, "TopHeader");
            }


            // Build Row Header
            foreach (PivotRowHeaderCell cell in pivotGrid.GridLayout.RowHeaderCells)
            {
                ColumnSpan = cell.ColumnSpan;
                ColumnId = pivotGrid.GridLayout.PrivateColumnsForRowPanel.IndexOf(cell.Column as PivotHeaderColumn);

                RowSpan = cell.RealRowSpan;
                RowId = pivotGrid.GridLayout.Rows.IndexOf(cell.Row as PivotDataRow) + TopHeaderHeight + currentRow;

                CellValue = cell.Member.Caption;
                if (cell.Member.IsTotal && cell.IsToggleVisible == false)
                    CellValue += " " + "Total";

                if (ColumnSpan > 1 || RowSpan > 1)
                {
                    sheet.MergedCellsRegions.Add(
                        RowId,
                        ColumnId,
                        RowId + RowSpan - 1,
                        ColumnId + ColumnSpan - 1
                        );
                }

                this.SetCellValue(sheet.Rows[RowId].Cells[ColumnId], CellValue, "LeftHeader");
            }

            // Build Data
            foreach (PivotDataRow row in pivotGrid.GridLayout.Rows)
            {
                int currentCell = 0;
                foreach (PivotCell cell in row.Cells)
                {
                    string currentCellValue;
                    if (cell.Data != null)
                    {
                        currentCellValue = (cell.Data as ICell).FormattedValue;
                    }
                    else
                    {
                        currentCellValue = string.Empty;
                    }
                    this.SetCellValue(
                        sheet.Rows[currentRow + TopHeaderHeight]
                            .Cells[currentCell + LeftHeaderWidth],
                        currentCellValue,
                        "Data");
                    currentCell++;
                }
                currentRow++;
            }

            if (CompactLayoutEnabled)
                pivotGrid.AllowCompactLayout = true;

            //return the filled in workbook
            return dataWorkbook;

        }

        //Set the value and format each cell
        private void SetCellValue(WorksheetCell cell, string cellValue, string cellType)
        {
            cell.Value = cellValue;
            cell.CellFormat.ShrinkToFit = ExcelDefaultableBoolean.True;
            cell.CellFormat.VerticalAlignment = VerticalCellAlignment.Center;
            cell.CellFormat.Alignment = HorizontalCellAlignment.Center;

            if (cellType == "TopHeader")
            {
                cell.CellFormat.Fill = CellFill.CreatePatternFill(GetColor(255, 181, 212, 240), GetColor(255, 255, 255, 255), FillPatternStyle.Solid);
            }
            else if (cellType == "LeftHeader")
            {
                cell.CellFormat.Fill = CellFill.CreatePatternFill(GetColor(255, 200, 212, 240), GetColor(255, 255, 255, 255), FillPatternStyle.Solid);
            }
            else if (cellType == "Data")
            {
                cell.CellFormat.Fill = CellFill.CreatePatternFill(GetColor(255, 242, 244, 249), GetColor(255, 255, 255, 255), FillPatternStyle.Solid);
            }
            else
            {
                MessageBox.Show("Not located");
            }
        }

        private Color GetColor(int a, int r, int g, int b)
        {
            return Color.FromArgb(a, r, g, b);
        }

        //Open a dialog and prompt the user to save the Excel workbook with the exported data
        private void SaveExport(Workbook dataWorkbook)
        {
            string fileName = Path.GetTempFileName();
            fileName = fileName.Replace(".tmp", ".xlsx");
            dataWorkbook.SetCurrentFormat(WorkbookFormat.Excel2007);

            using (Stream stream = File.Open(fileName, FileMode.OpenOrCreate))
            {
                dataWorkbook.Save(stream);
                stream.Close();
            }
            
            System.Diagnostics.Process.Start(fileName);
        }

        #endregion
    }
}
